"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.App = void 0;
const koa_body_1 = __importDefault(require("koa-body"));
const koa_1 = __importDefault(require("koa"));
const readdirp_1 = __importDefault(require("readdirp"));
const path_1 = __importDefault(require("path"));
class App {
    constructor(option) {
        this.cachedRouter = {};
        const defaultConfig = {
            proxy: true,
            port: 9000,
            defaultAction: 'index',
            beforeAction: 'beforeAction',
            bodyOptions: { multipart: true },
            onStart() { },
        };
        this.config = Object.assign(Object.assign({}, defaultConfig), option);
    }
    detectAllRouter() {
        var _a, e_1, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.config.controllerRoot) {
                throw new Error('Controller root directory does not exist');
            }
            const scanDir = path_1.default.normalize(this.config.controllerRoot);
            const filter = { fileFilter: ['*.js', '*.mjs', '*.cjs'] };
            try {
                for (var _d = true, _e = __asyncValues((0, readdirp_1.default)(scanDir, filter)), _f; _f = yield _e.next(), _a = _f.done, !_a;) {
                    _c = _f.value;
                    _d = false;
                    try {
                        const entry = _c;
                        const pathname = entry.path
                            .replace(/\.js$/i, '')
                            .replace(/\\/g, '/')
                            .toLowerCase();
                        let Class = require(entry.fullPath);
                        if (typeof Class === 'object') {
                            const keys = Object.keys(Class);
                            for (let key of keys) {
                                if (typeof Class[key] === 'function') {
                                    Class = Class[key];
                                    break;
                                }
                            }
                        }
                        if (typeof Class !== 'function') {
                            continue;
                        }
                        const actions = Reflect.ownKeys(Class.prototype);
                        for (const actionName of actions) {
                            if (actionName === 'constructor' ||
                                actionName === this.config.beforeAction ||
                                typeof actionName === 'symbol') {
                                continue;
                            }
                            const routerData = {
                                class: Class,
                                actionName,
                            };
                            const actionUri = actionName.toLowerCase();
                            if (actionUri === this.config.defaultAction) {
                                this.cachedRouter[pathname] = routerData;
                            }
                            this.cachedRouter[`${pathname}/${actionUri}`] = routerData;
                        }
                    }
                    finally {
                        _d = true;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    run() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            yield ((_b = (_a = this.config).onStart) === null || _b === void 0 ? void 0 : _b.call(_a));
            yield this.detectAllRouter();
            const server = new koa_1.default();
            server.proxy = this.config.proxy;
            server.use((0, koa_body_1.default)(this.config.bodyOptions));
            const middlewares = this.config.middlewares;
            if (Array.isArray(middlewares) && middlewares.length > 0) {
                for (const middleware of middlewares) {
                    server.use(middleware);
                }
            }
            server.use((ctx) => __awaiter(this, void 0, void 0, function* () {
                const pathname = ctx.path
                    .replace(/^\/*|\/*$/g, '')
                    .trim()
                    .toLowerCase();
                const data = this.cachedRouter[pathname];
                if (!data) {
                    ctx.status = 404;
                    return;
                }
                ctx.status = 200;
                ctx.body = '';
                const controller = new data.class(ctx);
                const beforeAction = this.config.beforeAction;
                if (typeof controller[beforeAction] === 'function') {
                    const beforeActionReturn = yield controller[beforeAction]();
                    if (!!beforeActionReturn) {
                        return;
                    }
                }
                yield controller[data.actionName]();
            }));
            server.listen(this.config.port);
            console.log(`Application started and listening on port :${this.config.port}`);
        });
    }
}
exports.App = App;
