"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collection = void 0;
class Collection {
    constructor(name, db) {
        this.name = name;
        this.db = db;
        this.instance = this.db.collection(name);
    }
    remove(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.instance.deleteMany(filter);
            return result.acknowledged;
        });
    }
    update(filter = {}, data = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.instance.updateMany(filter, { $set: data });
            return result.acknowledged;
        });
    }
    count(filter = {}, option = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.instance.countDocuments(filter, option);
        });
    }
    upsert(filter = {}, data = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.instance.updateOne(filter, { $set: data }, {
                upsert: true,
            });
            return result.acknowledged;
        });
    }
    add(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.instance.insertOne(data);
            if (result.acknowledged) {
                this.lastInsertedId = result.insertedId;
                return true;
            }
            return false;
        });
    }
    adds(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!Array.isArray(data)) {
                data = [data];
            }
            const result = yield this.instance.insertMany(data);
            if (result.acknowledged && result.insertedCount > 0) {
                this.lastInsertedIds = result.insertedIds;
                return true;
            }
            return false;
        });
    }
    find(filter = {}, option = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const cursor = yield this.instance.find(filter, option);
            return cursor.toArray();
        });
    }
    get(filter = {}, sort = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.instance.findOne(filter, { sort });
        });
    }
    gets(filter = {}, sort = {}, offset = 0, limit = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const option = {
                sort,
                skip: offset,
            };
            if (limit > 0) {
                option.limit = limit;
            }
            const cursor = yield this.instance.find(filter, option);
            return cursor.toArray();
        });
    }
}
exports.Collection = Collection;
