"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Loader = void 0;
const path_1 = __importDefault(require("path"));
class Loader {
    constructor(basePath) {
        this.basePath = [];
        this.cachedClass = {};
        if (typeof basePath === 'string') {
            this.basePath.push(basePath);
        }
        else if (Array.isArray(basePath)) {
            this.basePath.push(...basePath);
        }
    }
    resolve(classPath, ...params) {
        const cache = this.cachedClass[classPath];
        if (cache) {
            return new cache(...params);
        }
        const Class = require(classPath);
        if (typeof Class === 'function') {
            this.cachedClass[classPath] = Class;
            return new Class(...params);
        }
        if (typeof Class === 'object') {
            const keys = Object.keys(Class);
            for (let key of keys) {
                if (typeof Class[key] === 'function') {
                    this.cachedClass[classPath] = Class[key];
                    return new Class[key](...params);
                }
            }
        }
    }
    load(classPath, ...params) {
        const fullPaths = this.basePath.map((base) => {
            return path_1.default.resolve(base, classPath);
        });
        for (let fullPath of fullPaths) {
            const result = this.resolve(fullPath, ...params);
            if (result) {
                return result;
            }
        }
        throw new Error(`The loader cannot find the corresponding class：${classPath}`);
    }
}
exports.Loader = Loader;
