"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SQLParser = void 0;
class SQLParser {
    createOrder(order) {
        if (!order) {
            return '';
        }
        let parts = [];
        for (let key in order) {
            let type = order[key].toUpperCase();
            if (['ASC', 'DESC'].includes(type)) {
                parts.push(`\`${key}\` ${type}`);
            }
        }
        return parts.join(', ');
    }
    createFilter(key, value) {
        const pattern = /^\s*(.*?)\s*(\<[=\>]?|\>=?|(NOT\s+)?(BETWEEN|LIKE|IN|RLIKE))?\s*$/i;
        const found = key.match(pattern);
        let prepare = '';
        let holders = [];
        if (!found) {
            return { prepare, holders };
        }
        const field = found[1].trim();
        let op = found[2];
        if (!op) {
            prepare = `\`${field}\` = ?`;
            holders.push(value);
            return { prepare, holders };
        }
        op = op.trim().replace(/\s+/, ' ').toUpperCase();
        if (op === 'IN' || op === 'NOT IN') {
            prepare = `\`${field}\` ${op} `;
            let createInPair = (list) => {
                let places = [];
                let holders = [];
                for (let item of list) {
                    places.push('?');
                    holders.push(item);
                }
                return {
                    prepare: places.join(', '),
                    holders,
                };
            };
            let list = undefined;
            if (typeof value === 'string') {
                value = value.replace(/\s*\,\s*/g, ',');
                list = value.split(',');
            }
            else if (Array.isArray(value) && value.length > 0) {
                list = value;
            }
            if (list) {
                let result = createInPair(list);
                prepare += `(${result.prepare})`;
                holders.push(...result.holders);
            }
            else {
                prepare = '';
            }
            return { prepare, holders };
        }
        if (op === 'BETWEEN' || op === 'NOT BETWEEN') {
            if (Array.isArray(value) && value.length === 2) {
                prepare = `\`${field}\` ${op} ? AND ?`;
                holders = value;
            }
            return { prepare, holders };
        }
        prepare = `\`${field}\` ${op} ?`;
        holders.push(value);
        return { prepare, holders };
    }
    createDataFilter(key, value) {
        let prepare = '';
        let holders = [];
        const found = key.match(/^\s*(.+?)\s*([\+\-\*\/]=)?\s*$/);
        if (!found) {
            return { prepare, holders };
        }
        let field = found[1].trim();
        if (!found[2]) {
            prepare = `\`${field}\` = ?`;
            holders.push(value);
            return { prepare, holders };
        }
        let op = found[2].trim()[0];
        prepare = `\`${field}\` = \`${field}\` ${op} ?`;
        holders.push(value);
        return { prepare, holders };
    }
    createWhere(where) {
        let prepares = [];
        let holders = [];
        if (!where) {
            return { prepare: '', holders };
        }
        for (let key in where) {
            let value = where[key];
            let tmpHolders = this.createFilter(key, value);
            if (tmpHolders.prepare === '') {
                continue;
            }
            prepares.push(tmpHolders.prepare);
            holders.push(...tmpHolders.holders);
        }
        return {
            prepare: prepares.join(' AND '),
            holders,
        };
    }
}
exports.SQLParser = SQLParser;
