"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Table_instances, _Table_debug;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Table = void 0;
const SQLParser_1 = require("./SQLParser");
class Table extends SQLParser_1.SQLParser {
    constructor(name, db) {
        super();
        this.name = name;
        this.db = db;
        _Table_instances.add(this);
        this.lastInsertedId = undefined;
    }
    get(where, order, field = '*') {
        return __awaiter(this, void 0, void 0, function* () {
            let result = yield this.gets(where, order, 0, 1, field);
            return result.length > 0 ? result[0] : null;
        });
    }
    gets(where, order, offset = 0, limit = 0, field = '*') {
        return __awaiter(this, void 0, void 0, function* () {
            let sql = `SELECT ${field} FROM \`${this.name}\``;
            let whereResult = this.createWhere(where);
            if (whereResult.prepare) {
                sql += ` WHERE ${whereResult.prepare}`;
            }
            let orderStat = this.createOrder(order);
            if (orderStat) {
                sql += ` ORDER BY ${orderStat}`;
            }
            if (limit > 0) {
                sql += ` LIMIT ${offset}, ${limit}`;
            }
            try {
                __classPrivateFieldGet(this, _Table_instances, "m", _Table_debug).call(this, this.db.pool.format(sql, whereResult.holders));
                const result = yield this.db.pool.query(sql, whereResult.holders);
                return result[0];
            }
            catch (error) {
                console.error(error);
                return [];
            }
        });
    }
    add(data) {
        return __awaiter(this, void 0, void 0, function* () {
            let fields = [];
            let places = [];
            let holders = [];
            for (let key in data) {
                fields.push(`\`${key}\``);
                places.push('?');
                holders.push(data[key]);
            }
            let sql = `INSERT INTO \`${this.name}\` (${fields.join(', ')}) VALUES (${places.join(', ')})`;
            try {
                __classPrivateFieldGet(this, _Table_instances, "m", _Table_debug).call(this, this.db.pool.format(sql, holders));
                const result = yield this.db.pool.execute(sql, holders);
                const header = result[0];
                if (header.affectedRows !== 1) {
                    return false;
                }
                if (header.insertId > 0) {
                    this.lastInsertedId = header.insertId;
                }
                return true;
            }
            catch (error) {
                console.error(error);
                return false;
            }
        });
    }
    remove(where, order, limit = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            let sql = `DELETE FROM \`${this.name}\``;
            let whereResult = this.createWhere(where);
            if (whereResult.prepare) {
                sql += ` WHERE ${whereResult.prepare}`;
            }
            let orderStat = this.createOrder(order);
            if (orderStat) {
                sql += ` ORDER BY ${orderStat}`;
            }
            if (limit > 0) {
                sql += ` LIMIT ${limit}`;
            }
            try {
                __classPrivateFieldGet(this, _Table_instances, "m", _Table_debug).call(this, this.db.pool.format(sql, whereResult.holders));
                yield this.db.pool.execute(sql, whereResult.holders);
                return true;
            }
            catch (error) {
                console.error(error);
                return false;
            }
        });
    }
    count(where) {
        return __awaiter(this, void 0, void 0, function* () {
            let sql = `SELECT COUNT(*) as \`total_num\` FROM \`${this.name}\``;
            let whereResult = this.createWhere(where);
            if (whereResult.prepare) {
                sql += ` WHERE ${whereResult.prepare}`;
            }
            try {
                __classPrivateFieldGet(this, _Table_instances, "m", _Table_debug).call(this, this.db.pool.format(sql, whereResult.holders));
                let result = yield this.db.pool.query(sql, whereResult.holders);
                const list = result[0];
                return list.length === 0 ? 0 : list[0].total_num;
            }
            catch (error) {
                console.error(error);
                return 0;
            }
        });
    }
    update(data, where, order, limit = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            let parts = [];
            let holders = [];
            for (let key in data) {
                let result = this.createDataFilter(key, data[key]);
                if (!result.prepare) {
                    continue;
                }
                parts.push(result.prepare);
                holders.push(...result.holders);
            }
            let sql = `UPDATE \`${this.name}\` SET`;
            if (parts.length === 0) {
                return false;
            }
            sql += ` ${parts.join(', ')}`;
            let whereResult = this.createWhere(where);
            if (whereResult.prepare) {
                sql += ` WHERE ${whereResult.prepare}`;
                holders.push(...whereResult.holders);
            }
            let orderStat = this.createOrder(order);
            if (orderStat) {
                sql += ` ORDER BY ${orderStat}`;
            }
            if (limit > 0) {
                sql += ` LIMIT ${limit}`;
            }
            try {
                __classPrivateFieldGet(this, _Table_instances, "m", _Table_debug).call(this, this.db.pool.format(sql, holders));
                yield this.db.pool.execute(sql, holders);
                return true;
            }
            catch (error) {
                console.error(error);
                return false;
            }
        });
    }
    upsert(data, where) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!where) {
                return this.add(data);
            }
            const result = yield this.get(where);
            if (!result) {
                return this.add(data);
            }
            return this.update(data, where);
        });
    }
}
exports.Table = Table;
_Table_instances = new WeakSet(), _Table_debug = function _Table_debug(...params) {
    if (this.db.debug) {
        console.log(`[MySQL DEBUG]:`, ...params);
    }
};
